program MakeFat;

	uses
		Resources,
		MyInitialization, MyRequiredEventSupport, MySystemGlobals, MyUtils, MyFileSystemUtils, 
		MyStrings, MyEmergencyNotifier;
	
	var
		quitNow:Boolean;
		first:Boolean;
		fs1,fs2:FSSpec;
	
	type
		KindType = (K_Unknown, K_68k, K_PPC);
	
	procedure Debug(s:Str255);
	begin
		EmergencyNotify(s);
	end;
	
	function ProcessPair:OSErr;
		procedure Check(t:Boolean; msg:Str255);
		begin
			if not t then begin
				Debug(msg);
				Exit(ProcessPair);
			end;
		end;
		
		function FSpOpenResFileNoResLoad(var fs:FSSpec; perm:integer):integer;
		begin
			SetResLoad(false);
			FSpOpenResFileNoResLoad := FSpOpenResFile(fs, perm);
			SetResLoad(true);
		end;
		
		function Get1ResourceNoResLoad(typ:OSType; id:integer):Handle;
		begin
			SetResLoad(false);
			Get1ResourceNoResLoad:=Get1Resource(typ, id);
			SetResLoad(true);
		end;
		
		function Get1IndResourceNoResLoad(typ:OSType; index:integer):Handle;
		begin
			SetResLoad(false);
			Get1IndResourceNoResLoad:=Get1IndResource(typ, index);
			SetResLoad(true);
		end;
		
		procedure Compare(rn1, rn2:integer; reversed:Boolean);
			var
				typ:OSType;
				typ_index:integer;
				index:integer;
				h1, h2:Handle;
				the_type:OSType;
				the_name:Str255;
				id1, id2:integer;
				size1, size2:longInt;
				skip:Boolean;
		begin
			UseResFile(rn1);
			for typ_index := 1 to Count1Types do begin
				UseResFile(rn1);
				Get1IndType(typ,typ_index);
				for index := 1 to Count1Resources(typ) do begin
					UseResFile(rn1);
					h1 := Get1IndResourceNoResLoad(typ, index);
					GetResInfo(h1,id1,the_type,the_name);
					size1 := GetResourceSizeOnDisk(h1);
					ReleaseResource(h1);
					
					UseResFile(rn2);
					h2 := Get1ResourceNoResLoad(typ, id1);
					if h2 = nil then begin
						if reversed then begin
							skip := (typ = 'cfrg') & (id1 = 0);
						end else begin
							skip := (typ='CODE') or (typ='DATA');
						end;
						if not skip then begin
							Debug(concat(OSTypeToString(typ),'=',NumToStr(id1),' is missing'));
						end;
					end else begin
						GetResInfo(h2,id2,the_type,the_name);
						size2 := GetResourceSizeOnDisk(h2);
						ReleaseResource(h2);
						
						if (size1 <> size2) then begin
							Debug(concat(OSTypeToString(typ),'=',NumToStr(id1),' different sizes'));
						end;
					end;
				end;
			end;
		end;
		
		var
			name:Str63;
			tfs, dest:FSSpec;
			pb: CInfoPBRec;
			rn1, rn2, rnd:integer;
			h:Handle;
			dlen:longInt;
			err:OSErr;
	begin
		ProcessPair := -1;
		name := '';
		Check(not IsExtension(fs1.name,'FAT') & not IsExtension(fs2.name,'FAT'), 'No no, drop the 68k and PPC applications on to this, not the FAT application!');
		if IsExtension(fs1.name,'68k') then begin
			name := copy(fs1.name,1,length(fs1.name)-3);
		end else if IsExtension(fs2.name,'PPC') then begin
			name := copy(fs2.name,1,length(fs2.name)-3);
		end else if IsExtension(fs2.name,'68k') then begin
			tfs := fs1; fs1 := fs2; fs2 := tfs;
			name := copy(fs1.name,1,length(fs1.name)-3);
		end else if IsExtension(fs2.name,'PPC') then begin
			tfs := fs1; fs1 := fs2; fs2 := tfs;
			name := copy(fs2.name,1,length(fs2.name)-3);
		end;
		Check(name <> '', 'Couldnt find 68k or PPC in names');
		if (IUEqualString(name, fs1.name) = 0) or (IUEqualString(name, fs2.name) = 0) then begin
			name := concat(name,'FAT');
		end;
		if (IUEqualString(name, fs1.name) = 0) or (IUEqualString(name, fs2.name) = 0) then begin
			LimitStringLength(name,30,'');
			name := concat(name,'#');
		end;
		
		err := FSpGetCatInfo (fs1, 0, pb);
		Check(err = noErr, 'Cant get info on 68k file');
		Check(pb.ioFlLgLen = 0, '68k file has data fork');
		
		err := FSpGetCatInfo (fs2, 0, pb);
		Check(err = noErr, 'Cant get info on PPC file');
		Check(pb.ioFlLgLen <> 0, 'PPC file has no data fork');
		dlen := pb.ioFlLgLen;
		
		rn1:=FSpOpenResFileNoResLoad(fs1,fsRdPerm);
		Check(rn1 <> -1, 'Cant open 68k res file');
		h:=Get1ResourceNoResLoad('CODE',0);
		Check(h <> nil, '68k doesnt have a CODE=0');
		h:=Get1IndResourceNoResLoad('cfrg',1);
		Check(h = nil, '68k has a cfrg resource');


		rn2:=FSpOpenResFileNoResLoad(fs2,fsRdPerm);
		Check(rn2 <> -1, 'Cant open PPC res file');
		h:=Get1IndResourceNoResLoad('CODE',1);
		Check(h = nil, 'PPC has a CODE resource');
		h:=Get1ResourceNoResLoad('cfrg',0);
		Check(h <> nil, 'PPC doesnt have a cfrg=0');
		
		Compare(rn1,rn2,false);
		Compare(rn2,rn1,true);
		
		CloseResFile(rn1);
		CloseResFile(rn2);
		
		err:=FSMakeFSSpec(fs1.vRefNum,fs1.parID,name,dest);
		err:=FSpDelete(dest);
		err := DuplicateFile(fs1, dest);
		Check(err = noErr, 'Duplicate failed');
		
		err := FSpOpenDF(fs2, fsRdPerm, rn2);
		Check(err = noErr, 'Cant open PPC data fork');
		err := FSpOpenDF(dest, fsWrPerm, rnd);
		Check(err = noErr, 'Cant open FAT data fork');
		err := CopyData(rn2, rnd, dlen);
		Check(err = noErr, 'Copy data fork failed');
		err := FSClose(rn2);
		Check(err = noErr, 'Close source data fork failed');
		err := FSClose(rnd);
		Check(err = noErr, 'Close dest data fork failed');

		rn2:=FSpOpenResFileNoResLoad(fs2,fsRdPerm);
		Check(rn2 <> -1, 'Cant open PPC res file (2)');
		h:=Get1Resource('cfrg',0);
		Check(h <> nil, 'PPC doesnt have a cfrg=0');
		DetachResource(h);
		CloseResFile(rn2);
		
		rnd:=FSpOpenResFileNoResLoad(dest,fsRdWrPerm);
		Check(rnd <> -1, 'Cant open FAT res file (2)');
		AddResource(h,'cfrg',0,'');
		CloseResFile(rnd);
		ProcessPair := noErr;
	end;
	
	function DoQuit: OSErr;
	begin
		quitNow := true;
		DoQuit := noErr;
	end;
	
	function DoOApp: OSErr;
	begin
		quitNow := true;
		EmergencyNotify('No, no, drop the 68k and PPC applications on to me, and Ill make the FAT application');
		DoOApp := noErr;
	end;
	
	function DoODoc (var fs: FSSpec): OSErr;
	begin
		if first then begin
			fs1 := fs;
			DoODoc := noErr;
		end else begin
			fs2 := fs;
			DoODoc := ProcessPair;
			quitNow := true;
		end;
		first := not first;
	end;
	
	procedure WNE;
		var
			dummy:Boolean;
			er:EventRecord;
			err:OSErr;
	begin
		dummy:=WaitNextEvent(everyEvent,er,0,nil);
		if er.what = keyDown then begin
			quitNow := true;
		end;
		if er.what = kHighLevelEvent then begin
			err := AEProcessAppleEvent(er);
		end;
	end;

begin
	Initialization;
	InitSystemGlobals;;
	if system7 then begin
		InitAppleEvents (DoOApp, DoODoc, nil, DoQuit);
		first := true;
		quitNow:=false;
		while not quitNow do begin
			WNE;
		end;
	end;
end.
